(********************************************************************)
(*																	*)
(*						Basic Drawing Objects						*)
(*																	*)
(********************************************************************)

unit TClass;

interface

type TByteFile = File of Byte;

type TMap = array[0..11,0..7] of Byte;	{ Map array }

type TRoom = object
	RoomID:Byte;						{ Room number }
	EventStatic:Byte;					{ Event static flags }
	WeaponID:Byte;						{ Weapon required to kill }
	FastMonster:Boolean;				{ Fast or slow monster }
	ShootMonster:Boolean;				{ Shooting monster }
	HitsRequired:Byte;					{ Number of hits required }
	EventObject:Byte;					{ Object ID reqd for event tile }
	EventAction:Byte;					{ Event Action reqd }
	Exit:array[0..3] of Byte;			{ N,E,S,W exits }
	Map:TMap;							{ Tile map }
	Message:String;						{ Text Message }
	constructor Init;
	procedure	Load(ID:Integer);
	procedure	Save;
	function	FileName:String;
end;

type TTileGraphic = object				{ A Tile graphic }
	Pixels:array[0..7] of Byte;
	constructor	Init;
	procedure 	SetPixel(x,y:Integer;On:Boolean);
	function  	GetPixel(x,y:Integer):Boolean;
	procedure   Read(var f:TByteFile);
	procedure	Write(var f:TByteFile);
	procedure	Draw(x,y,Size,FCol:Integer);
end;

const	TileSetSize = 128;

type TTileSet = object					{ Set of 64 tiles }
	Tile:array[0..TileSetSize-1] of TTileGraphic;
	constructor Init;
	destructor	Done;
end;

type PRoom = ^TRoom;					{ Define pointers }
	 PTileGraphic = ^TTileGraphic;
     PTileSet = ^TTileSet;

implementation

uses Graph;

(************************************************************)
(*	Initialise the room object by setting everything to 0	*)
(************************************************************)

constructor TRoom.Init;
var x,y:Integer;
begin
	RoomID := 0;EventStatic := 0;
	WeaponID := 0;HitsRequired := 1;
	EventObject := 0;EventAction := 0;
	FastMonster := False;ShootMonster := False;
	Message := '';
	for x := 0 to 3 do Exit[x] := 0;
	for x := 0 to 11 do
		for y := 0 to 7 do Map[x,y] := 0;
end;

(********************************)
(*	Load the Room from Disk		*)
(********************************)

procedure	TRoom.Load(ID:Integer);
var f:File of Byte;
	x,y:Integer;
	c:Byte;
begin
	RoomID := ID;
	Assign(f,FileName);
	{$I-}
	Reset(f);
	{$I+}
	If IOResult = 0 then
		begin
			Read(f,RoomID,EventStatic,WeaponID);
			Read(f,HitsRequired,EventObject,EventAction);
			for x := 0 to 3 do Read(f,Exit[x]);
			for x := 0 to 11 do for y := 0 to 7 do Read(f,Map[x,y]);
			Message := '';
			if not eof(f) then
				repeat
					Read(f,c);
					if c <> 0 then Message := Concat(Message,Chr(c));
				until c = 0;
			if not eof(f) then
				begin
					Read(f,c);
					FastMonster := (c = 1);
				end;
			if not eof(f) then
				begin
					Read(f,c);
					ShootMonster := (c = 1);
				end;
			Close(f);
		end;
end;

(************************)
(*	Save file to disk	*)
(************************)

procedure	TRoom.Save;
var f:File of Byte;
	x,y:Integer;
	c:Byte;
begin
	Assign(f,FileName);
	Rewrite(f);
	Write(f,RoomID,EventStatic,WeaponID);
	Write(f,HitsRequired,EventObject,EventAction);
	for x := 0 to 3 do Write(f,Exit[x]);
	for x := 0 to 11 do for y := 0 to 7 do Write(f,Map[x,y]);
	for x := 1 to Length(Message) do
		begin
			c := Ord(Message[x]);
			Write(f,c);
        end;
	c := 0;
	Write(f,c);
	c := 0; if FastMonster then c := 1;
	Write(f,c);
	c := 0; if ShootMonster then c := 1;
	Write(f,c);
	Close(f);
end;

(********************************)
(*	Get filename for this room	*)
(********************************)

function 	TRoom.FileName:String;
var s:String;
begin
	Str(RoomID,s);
	FileName := Concat('ROOM',s,'.ARM');
end;

(****************************************)
(*	Initialise a tile graphic object	*)
(****************************************)

constructor	TTileGraphic.Init;
var i:Integer;
begin
	for i := 0 to 7 do Pixels[i] := 0;
end;

(****************************)
(*	Set a pixel on or off	*)
(****************************)

procedure TTileGraphic.SetPixel(x,y:Integer;On:Boolean);
begin
	x := (128 shr x);
	if On then
		Pixels[y] := Pixels[y] or x
	else
		Pixels[y] := Pixels[y] and (255 - x);
end;

(************************)
(*	Get a pixel value	*)
(************************)

function  TTileGraphic.GetPixel(x,y:Integer):Boolean;
begin
	GetPixel := (Pixels[y] and (128 shr x)) <> 0;
end;

(********************************************)
(*	Read and write the tile graphic data 	*)
(********************************************)

procedure   TTileGraphic.Read(var f:TByteFile);
var i:Integer;
begin
	for i := 0 to 7 do Pixels[i] := 0;
	if not eof(f) then
			for i := 0 to 7 do System.Read(f,Pixels[i]);
end;

procedure	TTileGraphic.Write(var f:TByteFile);
var i:Integer;
begin
	for i := 0 to 7 do System.Write(f,Pixels[i]);
end;

(********************************************)
(*	Display a tile graphic on the screen	*)
(********************************************)

procedure	TTileGraphic.Draw(x,y,Size,FCol:Integer);
var x1,y1,xc,yc,c:Integer;
begin
	if Size <= 4 then
		begin
			SetFillStyle(SolidFill,Black);
			Bar(x,y,x+8*Size-1,y+8*Size-1);
		end;

	for x1 := 0 to 7 do
		for y1 := 0 to 7 do
			if Size > 4 then
				begin
					xc := x + x1 * Size;yc := y + y1 * Size;
					SetColor(FCol);
					if not GetPixel(x1,y1) then SetColor(Black);
					Line(xc,yc,xc+Size-1,yc);
					Line(xc,yc,xc,yc+Size-1);
					SetColor(FCol);
					if GetPixel(x1,y1) then SetColor(Black);
					Line(xc+Size-1,yc+Size-1,xc,yc+Size-1);
					Line(xc+Size-1,yc+Size-1,xc+Size-1,yc);
				end
			else
				begin
                for xc := 0 to Size-1 do
					for yc := 0 to Size-1 do
						if GetPixel(x1,y1) then
							PutPixel(x+x1*Size+xc,y+y1*Size+yc,FCol);
				end;
end;

(********************************)
(* 	64 element tile set object	*)
(********************************)

constructor TTileSet.Init;
var i:Integer;
	f:TByteFile;
begin
	Assign(f,'TILE.BIN');
	Reset(f);
	for i := 0 to TileSetSize-1 do Tile[i].Init;
	for i := 0 to TileSetSize-1 do Tile[i].Read(f);
	Close(f);
end;

destructor  TTileSet.Done;
var i:Integer;
	f:TByteFile;
begin
	Assign(f,'TILE.BIN');
	Rewrite(f);
	for i := 0 to TileSetSize-1 do Tile[i].Write(f);
	Close(f);
end;

begin
end.